package pps.core.space.utils;

import pps.cloud.base.service.BasePowerLineCloudService;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.GetBasePowerLineViewOutput;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 常用服务调用工具类
 *
 * @author ZWT
 * @date 2023/09/05
 */
public class ServiceUtil {

    /*-------------------------服务调用-------------------------*/

    /**
     * 通过字典类型查询字典
     *
     * @param context 上下文
     * @param alias   别名
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getDictMap(XContext context, String alias) {
        QuerySysDictionaryViewInput dictInput = new QuerySysDictionaryViewInput();
        dictInput.setAlias(alias);
        dictInput.setPage(1);
        dictInput.setLimit(999);
        SystemDictionaryService dictionaryService = context.getBean(SystemDictionaryService.class);
        XListResult<QuerySysDictionaryViewOutput> dictPageResult = dictionaryService.queryChildSysDictionarysByParentAlias(context, dictInput);
        dictPageResult.throwIfFail();
        return dictPageResult.getResult().stream()
                .collect(Collectors.toMap(QuerySysDictionaryViewOutput::getDicKey, QuerySysDictionaryViewOutput::getDicName));
    }

    /**
     * 通过组织id获取系统组织名称
     *
     * @param context 上下文
     * @param ouId    组织id
     * @return {@link String}
     */
    public static String getSysOrgNameById(XContext context, String ouId) {
        SysOrganizationCloudService orgCloudService = context.getBean(SysOrganizationCloudService.class);
        GetSysOrganizationViewInput viewInput = new GetSysOrganizationViewInput();
        viewInput.setId(ouId);
        XSingleResult<GetSysOrganizationViewOutput> organization = orgCloudService.getSysOrganizationById(context, viewInput);
        organization.throwIfFail();
        return Optional.ofNullable(organization.getResult())
                .map(GetSysOrganizationViewOutput::getOuName)
                .orElse(null);
    }

    /**
     * 获取区域Map
     *
     * @param context 上下文
     * @return {@link Map}<{@link Integer}, {@link String}>
     */
    public static Map<Integer, String> getSysAreaMap(XContext context) {
        SystemAreaService systemAreaService = context.getBean(SystemAreaService.class);
        XListResult<GetSysAreaOutput> sysAreaList = systemAreaService.getSysAreaList(context, new GetSysAreaInput());
        sysAreaList.throwIfFail();
        return sysAreaList.getResult().stream()
                .collect(Collectors.toMap(GetSysAreaOutput::getId, GetSysAreaOutput::getName));
    }

    /**
     * 获取所有path包含组织ID的组织
     *
     * @param context 上下文
     * @param ouId    ou id
     * @return {@link List}<{@link String}>
     */
    public static List<String> getOrgIdsByPath(XContext context, String ouId) {
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        GetAllOuListByOuIdInput ouIdInput = new GetAllOuListByOuIdInput();
        ouIdInput.setOuId(ouId);
        XListResult<GetSysOrganizationViewOutput> allListByOuId = organizationCloudService.getAllListByOuId(context, ouIdInput);
        allListByOuId.throwIfFail();
        return allListByOuId.getResult().stream()
                .map(GetSysOrganizationViewOutput::getId)
                .collect(Collectors.toList());
    }

    /**
     * 通过组织机构ID集合获取详情列表
     *
     * @param context 上下文
     * @param ouIds   ou id
     * @return {@link Map}<{@link String}, {@link String}>
     */
    public static Map<String, String> getOuMapByOuIds(XContext context, List<String> ouIds) {
        GetOuListByOuIdsInput input = new GetOuListByOuIdsInput();
        input.setOuIdList(ouIds);
        SysOrganizationCloudService organizationCloudService = context.getBean(SysOrganizationCloudService.class);
        XListResult<GetOuListTreeOutput> ouListByOuIds = organizationCloudService.getOuListByOuIds(context, input);
        ouListByOuIds.throwIfFail();
        return ouListByOuIds.getResult().stream()
                .collect(Collectors.toMap(
                        GetOuListTreeOutput::getOuId, GetOuListTreeOutput::getOuName
                ));
    }

    /**
     * 通过ID查询线路名称
     *
     * @param context 上下文
     * @param lineId  线路id
     * @return {@link String}
     */
    public static String getPowerLineNameById(XContext context, String lineId) {
        BasePowerLineCloudService lineService = context.getBean(BasePowerLineCloudService.class);
        XSingleResult<GetBasePowerLineViewOutput> result = lineService.getBasePowerLineById(context, GetBasePowerLineInput.builder()
                .id(lineId)
                .build());
        result.throwIfFail();
        return Optional.ofNullable(result.getResult())
                .map(GetBasePowerLineViewOutput::getLineName)
                .orElse(null);
    }
}